@@ -20,9 +20,10 @@ import unittest
 import mock
 
 from opencensus.ext.azure import trace_exporter
+from opencensus.ext.azure.common.transport import TransportStatusCode
 from opencensus.trace.link import Link
 
-TEST_FOLDER = os.path.abspath('.test.exporter')
+TEST_FOLDER = os.path.abspath('.test.trace.exporter')
 
 
 def setUpModule():
@@ -40,12 +41,17 @@ def throw(exc_type, *args, **kwargs):
 
 
 class TestAzureExporter(unittest.TestCase):
+
+    def setUp(self):
+        os.environ["APPLICATIONINSIGHTS_STATSBEAT_DISABLED_ALL"] = "true"
+        return super(TestAzureExporter, self).setUp()
+
+    def tearDown(self):
+        del os.environ["APPLICATIONINSIGHTS_STATSBEAT_DISABLED_ALL"]
+        return super(TestAzureExporter, self).tearDown()
+
     def test_ctor(self):
-        from opencensus.ext.azure.common import Options
-        instrumentation_key = Options._default.instrumentation_key
-        Options._default.instrumentation_key = None
-        self.assertRaises(ValueError, lambda: trace_exporter.AzureExporter())
-        Options._default.instrumentation_key = instrumentation_key
+        self.assertRaises(ValueError, lambda: trace_exporter.AzureExporter(connection_string="", instrumentation_key=""))  # noqa: E501
 
     def test_init_exporter_with_proxies(self):
         exporter = trace_exporter.AzureExporter(
@@ -95,14 +101,14 @@ class TestAzureExporter(unittest.TestCase):
         exporter._stop()
 
     @mock.patch('opencensus.ext.azure.trace_exporter.AzureExporter.span_data_to_envelope')  # noqa: E501
-    def test_emit_failure(self, span_data_to_envelope_mock):
+    def test_emit_retry(self, span_data_to_envelope_mock):
         span_data_to_envelope_mock.return_value = ['bar']
         exporter = trace_exporter.AzureExporter(
             instrumentation_key='12345678-1234-5678-abcd-12345678abcd',
             storage_path=os.path.join(TEST_FOLDER, self.id()),
         )
         with mock.patch('opencensus.ext.azure.trace_exporter.AzureExporter._transmit') as transmit:  # noqa: E501
-            transmit.return_value = 10
+            transmit.return_value = TransportStatusCode.RETRY
             exporter.emit(['foo'])
         self.assertEqual(len(os.listdir(exporter.storage.path)), 1)
         self.assertIsNone(exporter.storage.get())
@@ -117,7 +123,7 @@ class TestAzureExporter(unittest.TestCase):
             storage_path=os.path.join(TEST_FOLDER, self.id()),
         )
         with mock.patch('opencensus.ext.azure.trace_exporter.AzureExporter._transmit') as transmit:  # noqa: E501
-            transmit.return_value = 0
+            transmit.return_value = TransportStatusCode.SUCCESS
             exporter.emit([])
             exporter.emit(['foo'])
             self.assertEqual(len(os.listdir(exporter.storage.path)), 0)
@@ -139,7 +145,7 @@ class TestAzureExporter(unittest.TestCase):
         )
 
         # SpanKind.CLIENT HTTP
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -168,7 +174,7 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.CLIENT,
-        ))
+        )))
         self.assertEqual(
             envelope.iKey,
             '12345678-1234-5678-abcd-12345678abcd')
@@ -221,7 +227,7 @@ class TestAzureExporter(unittest.TestCase):
         )
 
         # SpanKind.CLIENT unknown type
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -243,7 +249,7 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.CLIENT,
-        ))
+        )))
         self.assertEqual(
             envelope.iKey,
             '12345678-1234-5678-abcd-12345678abcd')
@@ -276,7 +282,7 @@ class TestAzureExporter(unittest.TestCase):
             'RemoteDependencyData')
 
         # SpanKind.CLIENT missing method
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -302,7 +308,7 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.CLIENT,
-        ))
+        )))
         self.assertEqual(
             envelope.iKey,
             '12345678-1234-5678-abcd-12345678abcd')
@@ -344,7 +350,7 @@ class TestAzureExporter(unittest.TestCase):
             'RemoteDependencyData')
 
         # SpanKind.SERVER HTTP - 200 request
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -373,7 +379,7 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.SERVER,
-        ))
+        )))
         self.assertEqual(
             envelope.iKey,
             '12345678-1234-5678-abcd-12345678abcd')
@@ -421,7 +427,7 @@ class TestAzureExporter(unittest.TestCase):
             'RequestData')
 
         # SpanKind.SERVER HTTP - Failed request
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -450,7 +456,102 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.SERVER,
+        )))
+        self.assertEqual(
+            envelope.iKey,
+            '12345678-1234-5678-abcd-12345678abcd')
+        self.assertEqual(
+            envelope.name,
+            'Microsoft.ApplicationInsights.Request')
+        self.assertEqual(
+            envelope.tags['ai.operation.parentId'],
+            '6e0c63257de34c93')
+        self.assertEqual(
+            envelope.tags['ai.operation.id'],
+            '6e0c63257de34c90bf9efcd03927272e')
+        self.assertEqual(
+            envelope.tags['ai.operation.name'],
+            'GET /wiki/Rabbit')
+        self.assertEqual(
+            envelope.time,
+            '2010-10-24T07:28:38.123456Z')
+        self.assertEqual(
+            envelope.data.baseData.id,
+            '6e0c63257de34c92')
+        self.assertEqual(
+            envelope.data.baseData.duration,
+            '0.00:00:00.111')
+        self.assertEqual(
+            envelope.data.baseData.responseCode,
+            '400')
+        self.assertEqual(
+            envelope.data.baseData.name,
+            'GET /wiki/Rabbit')
+        self.assertEqual(
+            envelope.data.baseData.success,
+            False)
+        self.assertEqual(
+            envelope.data.baseData.url,
+            'https://www.wikipedia.org/wiki/Rabbit')
+        self.assertEqual(
+            envelope.data.baseType,
+            'RequestData')
+
+        # SpanKind.SERVER HTTP - with exceptions
+        envelopes = exporter.span_data_to_envelope(SpanData(
+            name='test',
+            context=SpanContext(
+                trace_id='6e0c63257de34c90bf9efcd03927272e',
+                span_id='6e0c63257de34c91',
+                trace_options=TraceOptions('1'),
+                tracestate=Tracestate(),
+                from_header=False,
+            ),
+            span_id='6e0c63257de34c92',
+            parent_span_id='6e0c63257de34c93',
+            attributes={
+                'component': 'HTTP',
+                'http.method': 'GET',
+                'http.path': '/wiki/Rabbit',
+                'http.route': '/wiki/Rabbit',
+                'http.url': 'https://www.wikipedia.org/wiki/Rabbit',
+                'http.status_code': 400,
+                'error.message': 'bork bork',
+                'error.name': 'ValueError',
+            },
+            start_time='2010-10-24T07:28:38.123456Z',
+            end_time='2010-10-24T07:28:38.234567Z',
+            stack_trace=None,
+            links=None,
+            status=Status(0),
+            annotations=None,
+            message_events=None,
+            same_process_as_parent_span=None,
+            child_span_count=None,
+            span_kind=SpanKind.SERVER,
         ))
+
+        envelope = next(envelopes)
+        self.assertEqual(
+            envelope.iKey,
+            '12345678-1234-5678-abcd-12345678abcd')
+        self.assertEqual(
+            envelope.name,
+            'Microsoft.ApplicationInsights.Exception')
+        self.assertEqual(
+            envelope.tags['ai.operation.parentId'],
+            '6e0c63257de34c93')
+        self.assertEqual(
+            envelope.tags['ai.operation.id'],
+            '6e0c63257de34c90bf9efcd03927272e')
+        self.assertEqual(
+            envelope.time,
+            '2010-10-24T07:28:38.123456Z')
+        self.assertEqual(
+            envelope.data.baseType,
+            'ExceptionData')
+
+        envelope = next(envelopes)
         self.assertEqual(
             envelope.iKey,
             '12345678-1234-5678-abcd-12345678abcd')
@@ -492,7 +593,7 @@ class TestAzureExporter(unittest.TestCase):
             'RequestData')
 
         # SpanKind.SERVER unknown type
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -514,7 +615,7 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.SERVER,
-        ))
+        )))
         self.assertEqual(
             envelope.iKey,
             '12345678-1234-5678-abcd-12345678abcd')
@@ -541,7 +642,7 @@ class TestAzureExporter(unittest.TestCase):
             'RequestData')
 
         # SpanKind.UNSPECIFIED
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -563,7 +664,7 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.UNSPECIFIED,
-        ))
+        )))
         self.assertEqual(
             envelope.iKey,
             '12345678-1234-5678-abcd-12345678abcd')
@@ -600,7 +701,7 @@ class TestAzureExporter(unittest.TestCase):
             'RemoteDependencyData')
 
         # Status server status code attribute
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -624,12 +725,12 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.SERVER,
-        ))
+        )))
         self.assertEqual(envelope.data.baseData.responseCode, "201")
         self.assertTrue(envelope.data.baseData.success)
 
         # Status server status code attribute missing
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -651,11 +752,11 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.SERVER,
-        ))
+        )))
         self.assertFalse(envelope.data.baseData.success)
 
         # Server route attribute missing
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -683,12 +784,12 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.SERVER,
-        ))
+        )))
         self.assertEqual(envelope.data.baseData.properties['request.name'],
                          'GET /wiki/Rabbitz')
 
         # Server route and path attribute missing
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -715,12 +816,12 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.SERVER,
-        ))
+        )))
         self.assertIsNone(
             envelope.data.baseData.properties.get('request.name'))
 
         # Status client status code attribute
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -744,12 +845,12 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.CLIENT,
-        ))
+        )))
         self.assertEqual(envelope.data.baseData.resultCode, "201")
         self.assertTrue(envelope.data.baseData.success)
 
         # Status client status code attributes missing
-        envelope = exporter.span_data_to_envelope(SpanData(
+        envelope = next(exporter.span_data_to_envelope(SpanData(
             name='test',
             context=SpanContext(
                 trace_id='6e0c63257de34c90bf9efcd03927272e',
@@ -771,7 +872,7 @@ class TestAzureExporter(unittest.TestCase):
             same_process_as_parent_span=None,
             child_span_count=None,
             span_kind=SpanKind.CLIENT,
-        ))
+        )))
         self.assertFalse(envelope.data.baseData.success)
 
         exporter._stop()
